
#include <stdio.h>
#include <stdlib.h>
#include "cell.h"
#include "rif.h"
#include "rif_rgb.h"
#include "sif.h"
#include "sif_rgb.h"

extern void	Error(),RangeList();

#define LEN	1024

static double	Read(path,Nx,Ny,cell)
char		*path;
int		Nx,Ny;
Cell		***cell;
{
	int	x,y,bps;
	char	msg[LEN];

	ReadImageFile_RGB(path,&x,&y,&bps,cell+0,cell+1,cell+2,NULL);

	if (x!=Nx || y!=Ny) {
	    (void)sprintf(msg,"%s : bad image size.",path); Error(msg);
	}
	return 255.0/(double)((1<<bps)-1);
}

static Cell	Scan(arg)
char		*arg;
{
	unsigned int	ui;
	char		msg[LEN];

	if (sscanf(arg,"%u",&ui)!=1 || ui>255) {
	    (void)sprintf(msg,"%s : bad pixel value.",arg); Error(msg);
	}
	return (Cell)ui;
}

#define PV	(size_t)(Cell)(~0)

int	main(argc,argv)
int	argc;
char	**argv;
{
	int	Nx,Ny,y,x,gray=1;
	Cell	**cell,**bC[3],**fC[3],**R,**G,**B,r,g,b,bV[3],fV[3];
	char	pv01[PV+1];
	double	bR,fR;

	if (argc<6 || argc==7 || argc==9 || argc>10 ||
	   (argc==8 && (argv[3][0]=='-' || argv[4][0]!='-') &&
		       (argv[3][0]!='-' || argv[4][0]=='-')) ||
	   (argc==10 && (argv[3][0]!='-' || argv[6][0]!='-')))
	Error("usage : mask_RGB maskTIFF rangeList backTIFF|(-R G B) \\\n"
	      "                                    foreTIFF|(-R G B) newTIFF");

	ReadImageFile(argv[1],&Nx,&Ny,NULL,&cell,NULL);

	RangeList(argv[2],PV,pv01);

	if (argc==6) {
	    bR=Read(argv[3],Nx,Ny,bC);
	    fR=Read(argv[4],Nx,Ny,fC); R=fC[0]; G=fC[1]; B=fC[2];

	    for (y=0; y<Ny; y++) for (x=0; x<Nx; x++) {
		if (pv01[cell[y][x]]) {
		    r=R[y][x]=(Cell)(fR*(double)fC[0][y][x]+0.5);
		    g=G[y][x]=(Cell)(fR*(double)fC[1][y][x]+0.5);
		    b=B[y][x]=(Cell)(fR*(double)fC[2][y][x]+0.5);
		}
		else {
		    r=R[y][x]=(Cell)(bR*(double)bC[0][y][x]+0.5);
		    g=G[y][x]=(Cell)(bR*(double)bC[1][y][x]+0.5);
		    b=B[y][x]=(Cell)(bR*(double)bC[2][y][x]+0.5);
		}
		if (gray) gray=r==g && r==b;
	    }
	}
	else if (argc==10) {
	    bV[0]=Scan(argv[3]+1); bV[1]=Scan(argv[4]); bV[2]=Scan(argv[5]);
	    fV[0]=Scan(argv[6]+1); fV[1]=Scan(argv[7]); fV[2]=Scan(argv[8]);

	    R=cell;
	    if ((G=(Cell **)malloc(sizeof(Cell *)*(size_t)Ny))==NULL ||
		(B=(Cell **)malloc(sizeof(Cell *)*(size_t)Ny))==NULL ||
		(*G=(Cell *)malloc(sizeof(Cell)*(size_t)Ny*(size_t)Nx))==NULL ||
		(*B=(Cell *)malloc(sizeof(Cell)*(size_t)Ny*(size_t)Nx))==NULL)
		Error("memory allocation error.");

	    for (y=1; y<Ny; y++) {
		G[y]=G[y-1]+Nx; B[y]=B[y-1]+Nx;
	    }
	    for (y=0; y<Ny; y++) for (x=0; x<Nx; x++) {
		if (pv01[cell[y][x]]) {
		    r=R[y][x]=fV[0]; g=G[y][x]=fV[1]; b=B[y][x]=fV[2];
		}
		else {
		    r=R[y][x]=bV[0]; g=G[y][x]=bV[1]; b=B[y][x]=bV[2];
		}
		if (gray) gray=r==g && r==b;
	    }
	}
	else if (argv[3][0]!='-') {
	    bR=Read(argv[3],Nx,Ny,bC); R=bC[0]; G=bC[1]; B=bC[2];
	    fV[0]=Scan(argv[4]+1); fV[1]=Scan(argv[5]); fV[2]=Scan(argv[6]);

	    for (y=0; y<Ny; y++) for (x=0; x<Nx; x++) {
		if (pv01[cell[y][x]]) {
		    r=R[y][x]=fV[0]; g=G[y][x]=fV[1]; b=B[y][x]=fV[2];
		}
		else {
		    r=R[y][x]=(Cell)(bR*(double)bC[0][y][x]+0.5);
		    g=G[y][x]=(Cell)(bR*(double)bC[1][y][x]+0.5);
		    b=B[y][x]=(Cell)(bR*(double)bC[2][y][x]+0.5);
		}
		if (gray) gray=r==g && r==b;
	    }
	}
	else {
	    bV[0]=Scan(argv[3]+1); bV[1]=Scan(argv[4]); bV[2]=Scan(argv[5]);
	    fR=Read(argv[6],Nx,Ny,fC); R=fC[0]; G=fC[1]; B=fC[2];

	    for (y=0; y<Ny; y++) for (x=0; x<Nx; x++) {
		if (pv01[cell[y][x]]) {
		    r=R[y][x]=(Cell)(fR*(double)fC[0][y][x]+0.5);
		    g=G[y][x]=(Cell)(fR*(double)fC[1][y][x]+0.5);
		    b=B[y][x]=(Cell)(fR*(double)fC[2][y][x]+0.5);
		}
		else {
		    r=R[y][x]=bV[0]; g=G[y][x]=bV[1]; b=B[y][x]=bV[2];
		}
		if (gray) gray=r==g && r==b;
	    }
	}
	if (gray)
	    StoreImageFile(argv[argc-1],Nx,Ny,8,R,NULL);
	else
	    StoreImageFile_RGB(argv[argc-1],Nx,Ny,8,R,G,B,NULL);

	return 0;
}
