
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cell.h"
#include "csi.h"
#include "sif.h"

extern void	Error(),RangeList(),ReadSliceImage();

static Cell	**Alloc(Nx,Ny)
int		Nx,Ny;
{
	Cell	**cell;
	int	y;

	if ((cell=(Cell **)malloc(sizeof(Cell *)*(size_t)Ny))==NULL ||
	    (*cell=(Cell *)malloc(sizeof(Cell)*(size_t)Ny*(size_t)Nx))==NULL)
	    Error("memory allocation error.");

	for (y=1; y<Ny; y++) cell[y]=cell[y-1]+Nx;

	return cell;
}

#define LEN	1024

static char	**Check(dir,name,Nx,Ny,Nz)
char		*dir,*name;
int		Nx,Ny,Nz;
{
	char	**path,msg[LEN];
	int	x,y,z;

	path=CheckSliceImages(dir,name,&x,&y,&z,NULL);

	if (x!=Nx || y!=Ny || z!=Nz) {
	    (void)sprintf(msg,"%s/%s : bad image size.",dir,name); Error(msg);
	}
	return path;
}

#define PV	(size_t)(Cell)(~0)

static Cell	Scan(arg)
char		*arg;
{
	unsigned int	ui;
	char		msg[LEN];

	if (sscanf(arg+1,"%u",&ui)!=1 || ui>PV) {
	    (void)sprintf(msg,"%s : bad pixel value.",arg+1); Error(msg);
	}
	return (Cell)ui;
}

static void	Store(dir,name,largest,Nx,Ny,cell)
char		*dir,*name;
Cell		largest,**cell;
int		Nx,Ny;
{
	char	path[LEN];
	int	bps;

	(void)sprintf(path,"%s/%s",dir,name);
	for (bps=1; 1<<bps<=largest; bps<<=1) ;
	StoreImageFile(path,Nx,Ny,bps,cell,NULL);
}

int	main(argc,argv)
int	argc;
char	**argv;
{
	char	**path,pv01[PV+1],**bP,**fP;
	int	Nx,Ny,Nz,z,y,x;
	Cell	**cell,**bC,**fC,C,c,bV,fV;
	size_t	len;

	if (argc<7 || argc>9 ||
	   (argc==7 && (argv[4][0]!='-' || argv[5][0]!='-')) ||
	   (argc==8 && (argv[4][0]=='-' || argv[6][0]!='-') &&
		       (argv[4][0]!='-' || argv[5][0]=='-')))
	    Error(
"usage : si_mask maskDir nameFile rangeList (backDir nameFile)|-PV \\\n"
"                                           (foreDir nameFile)|-PV newDir"
	    );

	path=CheckSliceImages(argv[1],argv[2],&Nx,&Ny,&Nz,NULL);

	RangeList(argv[3],PV,pv01);

	cell=Alloc(Nx,Ny);

	len=strlen(argv[1])+1;

#define MASK(b,f)	ReadSliceImage(path[z],Nx,Ny,cell); \
			C=0; \
			for (y=0; y<Ny; y++) for (x=0; x<Nx; x++) \
			    if ((c=cell[y][x]=(pv01[cell[y][x]])?f:b)>C) C=c; \
			Store(argv[argc-1],path[z]+len,C,Nx,Ny,cell)

	if (argc==9) {
	    bP=Check(argv[4],argv[5],Nx,Ny,Nz); bC=Alloc(Nx,Ny);
	    fP=Check(argv[6],argv[7],Nx,Ny,Nz); fC=Alloc(Nx,Ny);
	    for (z=0; z<Nz; z++) {
		ReadSliceImage(fP[z],Nx,Ny,fC);
		ReadSliceImage(bP[z],Nx,Ny,bC); MASK(bC[y][x],fC[y][x]);
	    }
	}
	else if (argc==7) {
	    bV=Scan(argv[4]);
	    fV=Scan(argv[5]);
	    for (z=0; z<Nz; z++) {
		MASK(bV,fV);
	    }
	}
	else if (argv[4][0]!='-') {
	    bP=Check(argv[4],argv[5],Nx,Ny,Nz); bC=Alloc(Nx,Ny);
	    fV=Scan(argv[6]);
	    for (z=0; z<Nz; z++) {
		ReadSliceImage(bP[z],Nx,Ny,bC); MASK(bC[y][x],fV);
	    }
	}
	else {
	    bV=Scan(argv[4]);
	    fP=Check(argv[5],argv[6],Nx,Ny,Nz); fC=Alloc(Nx,Ny);
	    for (z=0; z<Nz; z++) {
		ReadSliceImage(fP[z],Nx,Ny,fC); MASK(bV,fC[y][x]);
	    }
	}
	return 0;
}
